function [depth,radiance] = ...
    initialize(I,maxDelta,win,gamma,F,D,p,costType)
% Compute initial depth and radiance with approx but efficient
% method
%
% Copyright 2006 Paolo Favaro (p.favaro@hw.ac.uk)
% 
% School of Engineering and Physical Sciences
% Heriot-Watt University, Edinburgh, UK
% 
% Last revision: August 2006
%
% This program can be used only for research purposes.
% This program is distributed WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.

[M,N,K] = size(I);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% fast initialization (only for 2 defocused images datasets)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Initializing depth map...   ');
NLevels = 50;
deltasigma = [0:maxDelta/(NLevels-1):maxDelta];
E = zeros(M,N,K,length(deltasigma));
W = 2*win+1;
WW = W*W;
Weights = WW*conv2(ones(M,N),ones(W)/W/W,'same');
Cext = zeros(M+2*W,N+2*W,K);
for k=1:size(I,3)
    [J,JJ(:,:,:,k)] = diffusion(I(:,:,k),maxDelta,NLevels-1,0);
end
for i=1:length(deltasigma)
    fprintf('\b\b\b\b[%2i]',i)
    J = squeeze(JJ(:,:,i,:));
    % choose either the l2 norm or the I-divergence
    if strcmp(costType,'l2')
        Cost = (I(:,:,end:-1:1)-J).^2;
    else
        Cost = I(:,:,end:-1:1).*log(I(:,:,end:-1:1)./J)-...
            I(:,:,end:-1:1)+J;
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    % original implementation 
    %Cost = convn(Cost,ones(W)/WW,'same');
    %E(:,:,:,i) = Cost./repmat(Weights,[1 1 K]);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % alternative implementation
    % use integral images instead of doing convolution 
    % this is faster when the kernel is rectangular and large
    Cext(W+[1:M],W+[1:N],:) = Cost;
    for k=1:K
        % compute integral image
        CC = cumsum(cumsum(Cext(:,:,k),1),2);
        E(:,:,k,i) = (CC(2*W-win:end-W+win,2*W-win:end-W+win)+...
            CC(W-win:end-2*W+win,W-win:end-2*W+win)-...
            CC(2*W-win:end-W+win,W-win:end-2*W+win)-...
            CC(W-win:end-2*W+win,2*W-win:end-W+win))./Weights;
    end
end
% search for minimum energy
for k=1:K
    [nrg(:,:,k),dd(:,:,k)] = min(E(:,:,k,:),[],4);
end
[dummy,kk] = min(nrg,[],3);
ind = zeros(M,N);
for k=1:K
    ind = ind+dd(:,:,k).*(kk==k);
end
sgn = (kk==1)-(kk==2);
% correct for discrepancies due to the integral images
sigma2 = .93*deltasigma(ind).*sgn;
% additional smoothing
for t=1:10
    if mod(t,2)==0
        sigma2 = sigma2 + 1e-1*laplacian(sigma2,'forward');
    else
        sigma2 = sigma2 + 1e-1*laplacian(sigma2,'backward');
    end
end
a = p(1)^2-p(2)^2;
b = -2*(p(1)-p(2));
c = sigma2/gamma^2/(D/2)^2;
fprintf('done\n');

% initialize depth map 
depth = 1./(1/F-(-b+sqrt(abs(b^2-4*a.*c)))/2/a);
% pick the sharpest regions and eliminate zeroes
radiance = I(:,:,2).*(kk==2)+I(:,:,1).*(kk==1); 

return