function [Z] = zerocrossing(I)
% Compute the zero crossings of I
%
% Copyright 2006 Paolo Favaro (p.favaro@hw.ac.uk)
% 
% School of Engineering and Physical Sciences
% Heriot-Watt University, Edinburgh, UK
% 
% Last revision: August 2006
%
% This program can be used only for research purposes.
% This program is distributed WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE.

[M,N] = size(I);
J = zeros(M+2,N+2);
J(1+[1:M],1+[1:N],1) = I;
J(1+[1:M],1+[1:N],2) = J([1:M],1+[1:N],1);
J(1+[1:M],1+[1:N],3) = J(2+[1:M],1+[1:N],1);
J(1+[1:M],1+[1:N],4) = J([1:M],[1:N],1);
J(1+[1:M],1+[1:N],5) = J(1+[1:M],[1:N],1);
J(1+[1:M],1+[1:N],6) = J(2+[1:M],[1:N],1);
J(1+[1:M],1+[1:N],7) = J([1:M],2+[1:N],1);
J(1+[1:M],1+[1:N],8) = J(1+[1:M],2+[1:N],1);
J(1+[1:M],1+[1:N],9) = J(2+[1:M],2+[1:N],1);
% positions in the matrix J of the 3x3 
% neighbourhood grid
% 4 2 7
% 5 1 8
% 6 3 9
J = J(1+[1:M],1+[1:N],:);
CrossTB = J(:,:,2).*J(:,:,3)<=0;
CrossLTDRBD = J(:,:,4).*J(:,:,9)<=0;
CrossLR = J(:,:,5).*J(:,:,8)<=0;
CrossLBDRTD = J(:,:,6).*J(:,:,7)<=0;
Z = (CrossTB|CrossLTDRBD|CrossLR|CrossLBDRTD);
% check for almost zero
Z = double(Z|(abs(I)<=1e-2));

return